#pragma once
#include <Fl/Fl.h>
#include <FL/Fl_Gl_Window.h>
#include <Fl/Fl_Double_Window.h>
#include <Fl/Fl_Button.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

// For subdivision...
//#include "SQRT3.h"
//#include "Loops.h"

extern int SHADING_STATE;


static GLfloat g_lightAmbient[]  =	{ 0.1f, 0.1f, 0.1f, 1.0f }; // Ambient Light (NEW)
static GLfloat g_lightDiffuse[]  =	{ 245.0/255.0, 164.0/255.0, 111.0/255.0, 1.0f }; // Fiffuse Light (NEW)
const GLfloat g_lightPosition1[] = {   .0f,1.0f, .0f, 0.0f }; // Light Source Position (NEW)
const GLfloat g_lightPosition2[] = { -1.0f, .0f, .0f, 0.0f }; // Light Source Position (NEW)
const GLfloat g_lightPosition3[] = {  1.0f, .0f, .0f, 0.0f }; // Light Source Position (NEW)
const GLfloat g_lightPosition4[] = {   .0f,-1.0f, .0f, 0.0f }; // Light Source Position (NEW)
// make a Gl window that draws something:
class MyGlWindow : public Fl_Gl_Window {
public:
	float RX, RY, RZ, TX, TY, TZ, SA;	//for opengl rotate, translate, scale
	int WIDTH, HEIGHT;					//window size
	float old_x, old_y;					//mouse event
	float Inv_Size;						//inverse of model bounding box size
	vector<Vector>	facet_normal;		//array of facet normal for flat-shading

	MyGlWindow(int x, int y, int w, int h) :	WIDTH(w), HEIGHT(h), 
												Fl_Gl_Window(x,y,w,h,"My GL Window"), 
												RX(0), RY(0), RZ(0),
												TX(0), TY(0), TZ(0), SA(1),
												old_x(0), old_y(0)
	{
	};
	void recompute_normal()				// should be called whenever mesh changed!
	{
	// This function is used to compute the surface normals.
	// Please complete this function and put the calculated normals to the vector facet_normal.
		facet_normal.clear();			// always remember clear the old dara.
	}
	void subdivide_sqrt3()
	{
		//call your sqrt3 subdivision function
//		sqrt3(P);
		recompute_normal();
	}
	void subdivide_loops()
	{
		//call your loops subdivision function
//		loops(P);
		recompute_normal();
	}

	void load_Mesh()
	{
		  if(P.size_of_vertices() != 0)		return;
		  OBJ_Reader<HalfedgeDS> reader;	//my Alias Wavefron .obj reader
		  P.delegate( reader );				// done.

		  ///////////////////////////////////////////////
		  // First, shift the entire mesh to the center

		  ///////////////////////////////////////////////
		  // Then, calculate the bounding box of the entire mesh

		  // After finding the bounding box, please replace the max_diag to it's largest length 
		  float max_diag = 1.0;

		  Inv_Size = 1.0/max_diag;

		  recompute_normal();
	}
	void save_Mesh()
	{
		if(P.size_of_vertices() == 0)		return;
		Write_OBJ(P);
	}

	int handle(int event) 
	  {
		  switch(event) 
		  {
		  case FL_PUSH:
			  {
				  old_x = Fl::event_x();
				  old_y = Fl::event_y();
			  }
			 return 1;
		  case FL_DRAG:
			  {
				  if(Fl::event_button() == 1)	//left
				  {
					RX += (old_y - Fl::event_y() )*0.3;
					RY += (old_x - Fl::event_x() )*0.3;
				  }
				  else
				  {
					  SA -= (old_y - Fl::event_y() )*0.005;
					  if(SA <= 0.1)	SA = 0.1;
				  }
				  
				  old_x = Fl::event_x();  old_y = Fl::event_y();
			  }
			  return 1;
		  default:
			  return Fl_Gl_Window::handle(event);
		  }
	  }

private:
	Polyhedron P;

	void init_Light()
	{
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
        glLightfv(GL_LIGHT1, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT1, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT1, GL_POSITION, g_lightPosition1); // Position The Light (NEW)
		glEnable( GL_LIGHT1);    
		glLightfv(GL_LIGHT2, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT2, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT2, GL_POSITION, g_lightPosition2); // Position The Light (NEW)
		glEnable( GL_LIGHT2);	
		glLightfv(GL_LIGHT3, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT3, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT3, GL_POSITION, g_lightPosition3); // Position The Light (NEW)
		glEnable( GL_LIGHT3);
		glLightfv(GL_LIGHT4, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT4, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT4, GL_POSITION, g_lightPosition4); // Position The Light (NEW)
		glEnable( GL_LIGHT4);
	}
	void draw() 
	{
		if (!valid()) {
			glClearDepth(1.0f);				// Depth Buffer Setup

			glEnable(GL_DEPTH_TEST);
			glDepthMask(GL_TRUE);
			glEnable(GL_CULL_FACE);
			glCullFace(GL_FRONT);
			glEnable(GL_NORMALIZE);
			glEnable(GL_POLYGON_SMOOTH);
			// glEnable(GL_LINE_SMOOTH);

			gluPerspective(45.0f,(GLfloat)WIDTH/(GLfloat)HEIGHT,-10.0f,-2000.0f);
			glViewport(0,0,WIDTH, HEIGHT);
			glClearColor(1,1,1,0);	

			init_Light();
			glMatrixMode(GL_MODELVIEW);     // Select The Modelview Matrix
			glLoadIdentity();				// Reset The Current Modelview Matrix
		}		

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
		glPushMatrix();
			glTranslatef(TX, TY, .0f);
			glRotatef(RX, 1.0f, 0.0f, 0.0f); glRotatef(RY, 0.0f, 1.0f, 0.0f); glRotatef(RZ, 0.0f, 0.0f, 1.0f);
			glScalef(Inv_Size*SA, Inv_Size*SA, Inv_Size*SA);
			// Start drawing!!
				
				if(SHADING_STATE == 1 || SHADING_STATE == 4)
				{
					glColor3f(0,0,0);
					glutWireCube(1);
				}
				if(SHADING_STATE == 2 || SHADING_STATE == 4)
				{
					glEnable(GL_LIGHTING);
					glutSolidCube(1);
					glDisable(GL_LIGHTING);			
				}
		glPopMatrix();
	};
};